# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/DashboardApi.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ai/h2o/featurestore/api/v1/DashboardApi.proto',
  package='ai.h2o.featurestore.api.v1',
  syntax='proto3',
  serialized_options=b'B\021DashboardApiProto',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n-ai/h2o/featurestore/api/v1/DashboardApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"a\n\x11\x44\x61shboardResponse\x12\x16\n\x0eproject_number\x18\x01 \x01(\x04\x12\x1b\n\x13\x66\x65\x61ture_sets_number\x18\x02 \x01(\x04\x12\x17\n\x0f\x66\x65\x61tures_number\x18\x03 \x01(\x04\x42\x13\x42\x11\x44\x61shboardApiProtob\x06proto3'
)




_DASHBOARDRESPONSE = _descriptor.Descriptor(
  name='DashboardResponse',
  full_name='ai.h2o.featurestore.api.v1.DashboardResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project_number', full_name='ai.h2o.featurestore.api.v1.DashboardResponse.project_number', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_sets_number', full_name='ai.h2o.featurestore.api.v1.DashboardResponse.feature_sets_number', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features_number', full_name='ai.h2o.featurestore.api.v1.DashboardResponse.features_number', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=77,
  serialized_end=174,
)

DESCRIPTOR.message_types_by_name['DashboardResponse'] = _DASHBOARDRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DashboardResponse = _reflection.GeneratedProtocolMessageType('DashboardResponse', (_message.Message,), {
  'DESCRIPTOR' : _DASHBOARDRESPONSE,
  '__module__' : 'ai.h2o.featurestore.api.v1.DashboardApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.DashboardResponse)
  })
_sym_db.RegisterMessage(DashboardResponse)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
